//
// db.c
// ausgelagerte DB-Funktionen
//
// GreyDNS - Greylisting DNS-Server
// Copyright (C) 2007  Robert Schulze (rob@rob-schulze.de)
// http://www.rob-schulze.de/publikationen/frickelworx/greydns
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
///////////////////////////////////////////////////////////////////////////////

#include "db.h"

#ifdef USE_QDBM

///////////////////////////////////////////////////////////////////////////////
// db_open
///////////////////////////////////////////////////////////////////////////////

DBTYPE *db_open(const char *dbname)
{
DBTYPE *db=dpopen(dbname,DP_OWRITER | DP_OCREAT,0);
if(db==NULL)
	{
	printf("unable to open db %s (%s)\n",dbname,dperrmsg(dpecode));
	if(dpecode==DP_EBROKEN)
		{
		printf("trying to reopen with DP_OTRUNC\n");
		db=dpopen(dbname,DP_OWRITER | DP_OTRUNC,0);
		if(db==NULL)
			{
			printf("unable to reopen db %s with DP_OTRUNC (%s)\n",dbname,dperrmsg(dpecode));
			return(NULL);
			}
		}
	}
printf("successfully opened %s (%d records)\n",dbname,dprnum(db));
return db;
}

///////////////////////////////////////////////////////////////////////////////
// db_get
// liefert 0 oder den Timestamp der Domain
///////////////////////////////////////////////////////////////////////////////

int db_get(DBTYPE *db, const char *domainname)
{
int match=0;

// erstmal gucken...
//
if(db==NULL)
	{
	printf("no valid dp-ptr!\n");
	return 0;
	}

char *m=dpget(db,domainname,-1,0,-1,NULL);
if(m==NULL)
	{
	return 0;
	}

match=atoi(m);
free(m);

return match;
}

///////////////////////////////////////////////////////////////////////////////
// db_set
// setzt den Timestamp der Domain
///////////////////////////////////////////////////////////////////////////////

int db_set(DBTYPE *db, const char *domainname, int stamp)
{
int result=0;

// erstmal gucken...
//
if(!db)
	{
	printf("no valid db-ptr!\n");
	return 0;
	}
else
	{
	// Zahl in String umbauen
	//
	char *stampstr=malloc(20);
	int strlen=sprintf(stampstr,"%-d",stamp);
	stampstr=realloc(stampstr,strlen+1);
	
	if(!dpput(db,domainname,-1,stampstr,strlen,DP_DOVER))
		{
		printf("db_set failed\n");
		}
	free(stampstr);
	}
return result;
}

///////////////////////////////////////////////////////////////////////////////
// db_close
// schliesst die DB
///////////////////////////////////////////////////////////////////////////////

void db_close(DBTYPE *db)
{
dpclose(db);
}

#endif /* USE_QDBM */

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

#ifdef USE_HASH

///////////////////////////////////////////////////////////////////////////////
// db_open
///////////////////////////////////////////////////////////////////////////////

DBTYPE *db_open(const char *dbname)
{
DBTYPE *db=hash_init(20000);
if(db==NULL)
	{
	printf("unable to create hash\n");
	return(NULL);
	}
	
return db;
}

///////////////////////////////////////////////////////////////////////////////
// db_get
// liefert 0 oder den Timestamp der Domain
///////////////////////////////////////////////////////////////////////////////

int db_get(DBTYPE *db, const char *domainname)
{
int match=0;

// erstmal gucken...
//
if(db==NULL)
	{
	printf("no valid dp-ptr!\n");
	return 0;
	}

struct btree_node *node=hash_get(db,domainname);
if(node==NULL)
	{
	return 0;
	}

match=node->stamp;

return match;
}

///////////////////////////////////////////////////////////////////////////////
// db_set
// setzt den Timestamp der Domain
///////////////////////////////////////////////////////////////////////////////

int db_set(DBTYPE *db, const char *domainname, int stamp)
{
int result=0;

// erstmal gucken...
//
if(!db)
	{
	printf("no valid db-ptr!\n");
	return 0;
	}
else
	{
	// Zahl in String umbauen
	//
	result=hash_set(db,domainname,stamp);
	}
return result;
}

///////////////////////////////////////////////////////////////////////////////
// db_close
// schliesst die DB
///////////////////////////////////////////////////////////////////////////////

void db_close(DBTYPE *db)
{
hash_free(db);
}

#endif

