//
// greydns.h
//
// GreyDNS - Greylisting DNS-Server
// Copyright (C) 2007  Robert Schulze (rob@rob-schulze.de)
// http://www.rob-schulze.de/publikationen/frickelworx/greydns
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
///////////////////////////////////////////////////////////////////////////////

#ifndef __greydns_h
#define __greydns_h

#include <stdio.h>
#include <stdlib.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <errno.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <signal.h>

#include "db.h"

// Port, auf dem greydns binden soll
//
const unsigned short PORT=5300;

// Adresse, auf der greydns lauschen soll
//
const char *LISTEN="127.0.0.1";

// Zeitraum, in dem eine Domain nach der ersten
// Anfrage grau gelistet ist
//
const unsigned short GREY_DELAY=10;

// Zeitraum, nach dem eine weiss gelistete
// Domain wieder grau gelistet wird
//
const unsigned short REMOVE_DELAY=60;

// Pfad zur DB
// wenn moeglich, in shared memory (tmpfs)
//
const char *DBNAME="/dev/shm/greydns.qdb";

// der TXT Record, der fuer eine grau gelistete
// Domain zurueckgegeben wird
//
const char *TXT_GREYLISTED="450 greylisted";

// der A Record, der fuer eine grau gelistete
// Domain zurueckgegeben wird
//
const char *A_GREYLISTED="127.0.0.2";

///////////////////////////////////////////////////////////////////////////////

char RUN=1;

const unsigned char SIZE_16=sizeof(u_int16_t);
const unsigned char SIZE_32=sizeof(u_int32_t);
const unsigned char SIZE_8=sizeof(unsigned char);
const unsigned char SIZE_TV=sizeof(struct timeval);


unsigned char TXT_GREYLISTED_LEN=0;
struct timeval SELECT_INTERVAL={1,0};
fd_set SELECT_FDS;
void *DNS_RR_IN_TXT=NULL;
void *DNS_RR_IN_A=NULL;
unsigned short SIZE_DNS_RR_IN_TXT=0;
unsigned short SIZE_DNS_RR_IN_A=0;

struct dns_header
	{
	u_int16_t serial;
	u_int16_t flags;
	u_int16_t qdcount;
	u_int16_t ancount;
	u_int16_t nscount;
	u_int16_t arcount;
	};

const unsigned char SIZE_HDR=sizeof(struct dns_header);

struct dns_question
	{
	char *qname;
	u_int16_t qtype;
	u_int16_t qclass;
	};

int sockbind(void);
void greydns(int, DBTYPE *);
void *create_answer(void *, int *, DBTYPE *);


struct dns_packet *unpack_request(const void *);
struct dns_header *unpack_request_header(const void *);
struct dns_question *unpack_request_question(const void *);


void *pack_response_nxdomain(void *, int *);
void modify_header_nxdomain(const void *);

void *pack_response_localhost(void *, char *, int *);
void modify_header_localhost(const void *);

void *pack_rr_in_a(const char *, int *);
void *pack_rr_in_txt(const char *, int *);
void *pack_domainname(const char *, int *);

void *prepack_rr_in_a(unsigned short *);
void *prepack_rr_in_txt(unsigned short *);

void sighandler_term();

#endif


